#!/usr/bin/env python
from pathlib import Path
import sys
from pyshortcuts import make_shortcut, uname, __file__

bindir = 'Scripts' if uname.startswith('win') else 'bin'

script = Path(sys.prefix, bindir, 'pyshortcut').resolve().as_posix()


iconfile = 'ladder.icns' if uname=='darwin' else 'shovel.ico'
icon = Path(Path(__file__).parent, 'icons', iconfile).absolute().as_posix()

scut = make_shortcut(f"{script} --wxgui", name='PyShortcuts',
                      icon=icon, terminal=False)

print("pyshortcuts GUI: %s" % scut.target)



#!/usr/bin/env python
"""
debug timer: measure run times of code, display results
"""
import time
from datetime import datetime

def isotime(dtime=None, sep=' '):
    "return datetime isoformat"
    if dtime is None:
        dtime = datetime.now()
    return datetime.isoformat(dtime, sep=sep)

class DebugTimer():
    '''
    Measure run times for lines of code and summarize results
    '''
    def __init__(self, title=None, verbose=False, precision=4):
        self.verbose = verbose
        if title is None:
            title = "DebugTimer"
        self.title = title
        self.precision = precision
        self.clear(title=title)

    def clear(self, title=None):
        """clear debugtimer"""
        self.data = []
        if title is None:
            title = self.title
        self.add('start')
        tparts = isotime().split('.')
        self.start_time = tparts[0] + '.' + tparts[1][:self.precision]

    def add(self, msg):
        "add message point to debugtimer"
        self.data.append((msg, time.perf_counter()))
        if self.verbose:
            print(msg)

    def get_report(self, precision=None):
        "get report text"
        if precision is None:
            precision = self.precision

        cols = [("Message", "Delta Time (s)", "Total Time (s)")]
        w0, w1, w2 = max(len(self.title), 32), 17, 17
        tlast = tstart = self.data[0][1]
        for msg, t in self.data:
            tt = f"{t-tstart:.{precision}f}"
            dt = f"{t-tlast:.{precision}f}"
            tlast = t
            cols.append((msg, dt, tt))
            w0 = max(len(msg)+1, w0)
            w1 = max(len(dt)+1,  w1)
            w2 = max(len(tt)+1,  w2)

        tline = f"+-{'-'*w0}+{'-'*w1}-+{'-'*w2}-+"
        out = [f"# {self.title:{w0}s}    {self.start_time:>{w1+w2}s}", tline]

        for i, col in enumerate(cols):
            out.append(f"| {col[0]:{w0}s}|{col[1]:>{w1}s} |{col[2]:>{w2}s} |")
            if i == 0:
                out.append(tline.replace('-', '='))
        out.append(tline)
        out.append('')

        return "\n".join(out)


    def show(self, precision=None, clear=True):
        "print report text"
        print(self.get_report(precision=precision))
        if clear:
            self.clear(title=self.title)


def debugtimer(title=None, precision=4, verbose=False):
    '''return a DebugTimer object to measure the run time of
    portions of code, and write a simple report.

    Arguments
    ------------
    title:      str, optional initial message ['DebugTimer']
    precision:  int, precision for timing results [4]
    verbose:    bool, whether to print() each message when entered [False]

    Returns
    --------
    DebugTimer object, with methods:

      clear(title=None)  : reset Timer
      add(message)       : record an event, saving message and time
      get_report()       : return text of report
      show()             : print out text of report and optionally clear

    Example:
    -------
      timer = debugtimer('timer testing', precision=4)
      result = foo(x=100)
      timer.add('ran function foo')
      bar(result)
      timer.add('ran bar')
      timer.show()
    '''
    return DebugTimer(title=title, precision=precision, verbose=verbose)



if __name__ == '__main__':
    import numpy as np
    dtimer = debugtimer('test timer')
    time.sleep(1.1010)
    dtimer.add('slept for 1.101 seconds')
    nx = 10_000_000
    x = np.arange(nx, dtype='float64')/3.0
    dtimer.add(f'created numpy array len={nx}')
    s = np.sqrt(x)
    dtimer.add('took sqrt')
    dtimer.show(precision=4)



import time
import os
import sys
import wx

import wx.lib.filebrowsebutton as filebrowse

from .utils import (fix_filename, get_cwd, uname)

make_shortcut =  get_folders = None

if uname.startswith('lin'):
    from .linux import make_shortcut, get_folders
elif uname.startswith('darwin'):
    from .darwin import make_shortcut, get_folders
elif uname.startswith('win'):
    from .windows import make_shortcut, get_folders

USERFOLDERS = get_folders()
DESKTOP = USERFOLDERS.desktop

PY_FILES = "Python scripts (*.py)|*.py"
ALL_FILES = "All files (*.*)|*.*"
ICO_FILES = "Icon files (*.ico)|*.ico"
ICNS_FILES = "Mac Icon files (*.icns)|*.icns"

CEN = wx.ALIGN_CENTER
LEFT = wx.ALIGN_LEFT
RIGHT = wx.ALIGN_RIGHT
ALL_CEN =  wx.ALL|CEN
ALL_LEFT =  wx.ALL|LEFT
ALL_RIGHT =  wx.ALL|RIGHT

FONTSIZE = 11
if uname == 'linux':
    FONTSIZE = 10

if uname.startswith('darwin'):
    wx.PyApp.IsDisplayAvailable = lambda _: True

class ShortcutFrame(wx.Frame):
    def __init__(self):

        wx.Frame.__init__(self, None, -1, 'Pyshortcuts Creator',
                          style=wx.DEFAULT_FRAME_STYLE|wx.RESIZE_BORDER|wx.TAB_TRAVERSAL)
        self.SetTitle('Pyshortcuts Creator')

        self.SetFont(wx.Font(FONTSIZE, wx.SWISS, wx.NORMAL, wx.BOLD, False))
        menu = wx.Menu()
        menu_exit = menu.Append(-1, "Q&uit", "Exit")

        menuBar = wx.MenuBar()
        menuBar.Append(menu, "&File")
        self.SetMenuBar(menuBar)

        self.Bind(wx.EVT_MENU, self.onExit, menu_exit)
        self.Bind(wx.EVT_CLOSE, self.onExit)

        panel = wx.Panel(self)

        opts = dict(size=(225, -1))
        lab_exec = wx.StaticText(panel, label=' Executable:', **opts)
        lab_script = wx.StaticText(panel, label=' Python Script:', **opts)
        lab_args  = wx.StaticText(panel, label=' Command-line Arguments:', **opts)
        lab_name  = wx.StaticText(panel, label=' Shortcut Name:', **opts)
        lab_desc  = wx.StaticText(panel, label=' Description:', **opts)
        lab_icon  = wx.StaticText(panel, label=' Icon File:', **opts)
        lab_folder = wx.StaticText(panel, label=' Desktop SubFolder:', **opts)
        lab_opts  = wx.StaticText(panel, label=' Options:', **opts)

        opts['size'] = (400, -1)

        sopts = dict(size=(400, -1), style=wx.TE_PROCESS_ENTER)

        self.txt_exec = wx.TextCtrl(panel, value=sys.executable, **sopts)
        self.txt_script = wx.TextCtrl(panel, value='', **sopts)
        self.txt_args = wx.TextCtrl(panel, value='', **opts)
        self.txt_name = wx.TextCtrl(panel, value='', **sopts)
        self.txt_desc = wx.TextCtrl(panel, value='', **opts)
        self.txt_icon = wx.TextCtrl(panel, value='', **opts)
        self.txt_folder= wx.TextCtrl(panel, value='', **opts)

        self.txt_script.Bind(wx.EVT_TEXT_ENTER, self.onScriptEnter)
        self.txt_name.Bind(wx.EVT_TEXT_ENTER, self.onNameEnter)

        self.opt_terminal = wx.CheckBox(panel, label='Run in Terminal?',
                                        size=(250, -1))
        self.opt_terminal.SetValue(1)

        targets = ('Desktop and Start Menu Shortcut',
                   'Desktop Shortcut only',
                   'Start Menu Shortcut only')

        self.targetchoice = wx.Choice(panel, choices=targets,
                                       size=(275, -1))
        self.targetchoice.SetSelection(0)
        self.targetchoice.Enable(uname!='darwin')

        btn_script = wx.Button(panel, -1, label='Browse', size=(100, -1))
        btn_script.Bind(wx.EVT_BUTTON, self.onBrowseScript)

        btn_icon = wx.Button(panel, -1, label='Browse', size=(100, -1))
        btn_icon.Bind(wx.EVT_BUTTON, self.onBrowseIcon)

        btn_folder = wx.Button(panel, -1, label='Browse', size=(100, -1))
        btn_folder.Bind(wx.EVT_BUTTON, self.onBrowseFolder)

        sizer = wx.GridBagSizer(5, 5)

        irow = 0
        sizer.Add(wx.StaticLine(panel, size=(650, 4)), (irow, 0), (1, 3), ALL_CEN)

        irow += 1
        sizer.Add(lab_exec,      (irow, 0), (1, 1), ALL_LEFT)
        sizer.Add(self.txt_exec, (irow, 1), (1, 1), ALL_LEFT)

        irow += 1
        sizer.Add(lab_script,      (irow, 0), (1, 1), ALL_LEFT)
        sizer.Add(self.txt_script, (irow, 1), (1, 1), ALL_LEFT)
        sizer.Add(btn_script,      (irow, 2), (1, 1), ALL_LEFT)

        irow += 1
        sizer.Add(lab_args,       (irow, 0), (1, 1), ALL_LEFT)
        sizer.Add(self.txt_args,  (irow, 1), (1, 1), ALL_LEFT)

        irow += 1
        sizer.Add(lab_name,       (irow, 0), (1, 1), ALL_LEFT)
        sizer.Add(self.txt_name,  (irow, 1), (1, 1), ALL_LEFT)

        irow += 1
        sizer.Add(lab_desc,       (irow, 0), (1, 1), ALL_LEFT)
        sizer.Add(self.txt_desc,  (irow, 1), (1, 1), ALL_LEFT)

        irow += 1
        sizer.Add(lab_icon,       (irow, 0), (1, 1), ALL_LEFT)
        sizer.Add(self.txt_icon,  (irow, 1), (1, 1), ALL_LEFT)
        sizer.Add(btn_icon,       (irow, 2), (1, 1), ALL_LEFT)

        irow += 1
        sizer.Add(lab_folder,      (irow, 0), (1, 1), ALL_LEFT)
        sizer.Add(self.txt_folder, (irow, 1), (1, 1), ALL_LEFT)
        sizer.Add(btn_folder,      (irow, 2), (1, 1), ALL_LEFT)

        irow += 1
        sizer.Add(lab_opts,         (irow, 0), (1, 1), ALL_LEFT)
        sizer.Add(self.targetchoice, (irow, 1), (1, 1), ALL_LEFT)
        sizer.Add(self.opt_terminal, (irow, 2), (1, 1), ALL_LEFT)

        irow += 1
        sizer.Add(wx.StaticLine(panel, size=(650, 4)), (irow, 0), (1, 3), ALL_CEN)

        btn_create = wx.Button(panel, label='Create Shortcut',  size=(175, -1))
        btn_create.Bind(wx.EVT_BUTTON, self.onCreate)

        btn_savepy = wx.Button(panel, label='Save Python Code', size=(175, -1))
        btn_savepy.Bind(wx.EVT_BUTTON, self.onSavePy)


        bsizer = wx.BoxSizer(wx.HORIZONTAL)
        bsizer.Add(btn_create, 1,  ALL_LEFT, 3)
        bsizer.Add(btn_savepy, 1,  ALL_LEFT, 3)

        irow += 1
        sizer.Add(bsizer, (irow, 0), (1, 3), ALL_LEFT)

        panel.SetSizer(sizer)
        sizer.Fit(panel)

        fsizer = wx.BoxSizer(wx.VERTICAL)
        fsizer.Add(panel, 0, wx.ALIGN_LEFT|wx.ALIGN_CENTER)
        fsizer.Fit(self)
        self.Refresh()

    def onBrowseScript(self, event=None):
        wildcards = "%s|%s" % (PY_FILES, ALL_FILES)

        dlg = wx.FileDialog(self, message='Select Python Script file',
                            wildcard=wildcards,
                            defaultDir=get_cwd(),
                            style=wx.FD_OPEN)

        if dlg.ShowModal() == wx.ID_OK:
            path = os.path.abspath(dlg.GetPath())
            self.txt_script.SetValue(path)

            _, name = os.path.split(path)
            name = fix_filename(name)
            if name.endswith('.py'):
                name = name[:-3]

            txt_name = self.txt_name.GetValue().strip()
            if len(txt_name) < 1:
                self.txt_name.SetValue(name)

            txt_desc = self.txt_desc.GetValue().strip()
            if len(txt_desc) < 1:
                self.txt_desc.SetValue(name)
        dlg.Destroy()


    def onBrowseIcon(self, event=None):

        wildcards = "%s|%s" % (ICO_FILES, ALL_FILES)
        if uname.startswith('darwin'):
            wildcards = "%s|%s" % (ICNS_FILES, ALL_FILES)

        dlg = wx.FileDialog(self, message='Select Icon file',
                            wildcard=wildcards,
                            defaultDir=get_cwd(),
                            style=wx.FD_OPEN)

        if dlg.ShowModal() == wx.ID_OK:
            path = os.path.abspath(dlg.GetPath())
            self.txt_icon.SetValue(path)
        dlg.Destroy()


    def onBrowseFolder(self, event=None):
        defdir = self.txt_folder.GetValue()
        if defdir in ('', 'Desktop'):
            defdir = DESKTOP
        dlg = wx.DirDialog(self,
                           message='Select Folder for Shortcut',
                           defaultPath=defdir,
                           style = wx.DD_DEFAULT_STYLE)

        if dlg.ShowModal() == wx.ID_OK:
            folder = os.path.abspath(dlg.GetPath())
            desktop = DESKTOP
            if folder.startswith(desktop):
                folder.replace(desktop, '')
                if folder.startswith('/'):
                    folder = folder[1:]
            self.txt_folder.SetValue(folder)
        dlg.Destroy()


    def onScriptEnter(self, event=None):
        path = self.txt_script.GetValue()

        _, name = os.path.split(path)
        name = fix_filename(name)
        if name.endswith('.py'):
            name = name[:-3]

        txt_name = self.txt_name.GetValue().strip()
        if len(txt_name) < 1:
            self.txt_name.SetValue(name)

        txt_desc = self.txt_desc.GetValue().strip()
        if len(txt_desc) < 1:
            self.txt_desc.SetValue(name)


    def onNameEnter(self, event=None):
        name = self.txt_name.GetValue()
        txt_desc = self.txt_desc.GetValue().strip()
        if len(txt_desc) < 1:
            self.txt_desc.SetValue(name)

    def read_form(self, as_string=False):
        def str_or_None(wid, as_string=False):
            val = wid.GetValue().strip()
            if len(val) < 1:
                val = None
            if as_string:
                val = 'None' if val is None else "'%s'" % val
            return val

        script = str_or_None(self.txt_script, as_string=as_string)
        args   = str_or_None(self.txt_args, as_string=as_string)
        name   = str_or_None(self.txt_name, as_string=as_string)
        desc   = str_or_None(self.txt_desc, as_string=as_string)
        icon   = str_or_None(self.txt_icon, as_string=as_string)
        folder = str_or_None(self.txt_folder, as_string=as_string)
        executable = str_or_None(self.txt_exec, as_string=as_string)

        targets = self.targetchoice.GetStringSelection().lower()
        desktop = 'desktop' in targets
        startmenu = 'start' in targets

        terminal = self.opt_terminal.IsChecked()

        if script in (None, 'None'):
            dlg = wx.MessageDialog(self, "script required",
                                   style=wx.OK|wx.ICON_INFORMATION)
            dlg.ShowModal()
            dlg.Destroy()
            return

        def cleanpath(val):
            if val not in (None, 'None'):
                val = val.replace('\\', '/').strip()
            return val

        script = cleanpath(script)
        folder = cleanpath(folder)
        icon = cleanpath(icon)
        executable = cleanpath(executable)

        if args not in (None, 'None'):
            if as_string:
                script = "'%s %s'" % (script[1:-1], args[1:-1])
            else:
                script = "%s %s" % (script, args)
        script = script.strip()
        return dict(script=script, name=name, description=desc, icon=icon,
                   folder=folder, terminal=terminal, desktop=desktop,
                   startmenu=startmenu, executable=executable)

    def onCreate(self, event=None):
        opts = self.read_form()
        if opts is None:
            return
        script = opts.pop('script')
        make_shortcut(script, **opts)

    def onSavePy(self, event=None):
        wildcards = "%s|%s" % (PY_FILES, ALL_FILES)
        dlg = wx.FileDialog(self,
                            message='Save Python script for creating shortcut',
                            defaultFile='make_shortcut.py',
                            wildcard=wildcards,
                            defaultDir=get_cwd(),
                            style=wx.FD_SAVE)

        if dlg.ShowModal() == wx.ID_OK:
            path = os.path.abspath(dlg.GetPath())
            opts = self.read_form(as_string=True)
            if opts is None:
                return
            buff = ['#!/usr/bin/env python',
                    'from pyshortcuts import make_shortcut',
                    """make_shortcut({script:s},
              name={name:s},
              description={description:s},
              folder={folder:s},
              icon={icon:s},
              terminal={terminal}, desktop={desktop}, startmenu={startmenu},
              executable={executable:s})""".format(**opts)]

            with open(path, 'w') as fh:
                fh.write('\n'.join(buff))


    def onAbout(self, event):
        dlg = wx.MessageDialog(self, "pyshortcuts Graphical User Interface",
                               style=wx.OK|wx.ICON_INFORMATION)
        dlg.ShowModal()
        dlg.Destroy()

    def onExit(self, event):
        self.Destroy()

if __name__ == '__main__':
    app = wx.App()
    ShortcutFrame().Show(True)
    app.MainLoop()



#!/usr/bin/env python
'''Normal test run:
     python -m pytests

   Run and see manual debug print()s:
     python test_pyshortcuts.py
'''
import os
import pytest
from pyshortcuts import make_shortcut, platform, Shortcut, get_folders

folders = get_folders()
root = os.path.abspath(os.path.join(__file__, '..', '..'))

def test_shortcut():
    '''Create a shortcut from examples dir'''
    print('---test_shortcut()')
    print("folders:\n Home: {}\n Desktop: {}\n StartMenu: {}\n".format(
        folders.home, folders.desktop, folders.startmenu))

    script = os.path.join(root, 'examples', 'console_scripts', 'timer.py') + ' -u 0.25 -t 10'
    icon  = os.path.join(root, 'examples', 'icons', 'stopwatch')

    iext = 'ico'
    if platform.startswith('darwin'):
        iext = 'icns'

    icon = "%s.%s" % (icon, iext)
    scut = make_shortcut(script, name='Timer', icon=icon, folder=folders.desktop)
    assert isinstance(scut, Shortcut), 'it returns a shortcut instance'

def test_none_script():
    '''Verify we get ValueError if cmd is None (#19)'''
    print('---test_none_script()')
    with pytest.raises(ValueError):
        cmd = None
        scut = make_shortcut(cmd, name="My Script is None")
    print('OK')

def test_empty_string_script():
    '''Verify we get ValueError if cmd is an empty string (#19)'''
    print('---test_empty_script()')
    with pytest.raises(ValueError):
        cmd = ""
        scut = make_shortcut(cmd, name="My Script is empty string")
    print('OK')

if __name__ == '__main__':
    test_shortcut()
    test_none_script()
    test_empty_string_script()



#!/usr/bin/env python
"""
This example shows how to make a shortcut that runs a single python command

Here, the comand to run is
     python -m pip install --upgrade pyshortcuts

to update to the latest version of this library.


Note that putting '_' or '{}' as the beginning of the command will tell
the script to use the current Python executable.
"""
from pathlib import Path
from pyshortcuts import make_shortcut, uname, get_cwd

pycmd = "_ -m pip install --upgrade pyshortcuts"

iconfile = 'shovel.icns' if uname=='darwin' else 'shovel.ico'
icon = Path(get_cwd(), 'icons', iconfile).absolute().as_posix()

make_shortcut(pycmd, name='Update Pyshortcuts', icon=icon)



#!/usr/bin/env python
"""
Create desktop shortcuts for Windows
"""
import os
import sys
import time
from pathlib import Path
from collections import namedtuple
import win32com.client
from win32com.shell import shell, shellcon

from .utils import get_pyexe, get_homedir

def get_conda_active_env():
    '''Return name of active conda environment or empty string'''
    c_env = None
    try:
        c_env = os.environ['CONDA_DEFAULT_ENV']
    except KeyError:
        print("No conda env active, defaulting to base")
        c_env = "base"
    return c_env

# batch file to activate the environment
# for Anaconda Python before running command.
conda_env = get_conda_active_env()
ENVRUNNER = r"""
@ECHO OFF
if exist "%~dp0..\Scripts\activate.bat" (
  call "%~dp0..\Scripts\activate.bat" {0}
) else (
   if exist "%~dp0..\condabin\conda.bat" (
      call "%~dp0..\condabin\conda.bat" activate {0}
   )
)
echo # run in conda environment "%CONDA_DEFAULT_ENV%":
echo # %*
%*
""".format(conda_env)


def get_desktop():
    '''Return user Desktop folder'''
    return shell.SHGetFolderPath(0, shellcon.CSIDL_DESKTOP, None, 0)

def get_startmenu():
    '''Return user Start Menu Programs folder
    note that we return CSIDL_PROGRAMS not CSIDL_COMMON_PROGRAMS
    '''
    return shell.SHGetFolderPath(0, shellcon.CSIDL_PROGRAMS, None, 0)

def get_folders():
    """get user-specific folders

    Returns:
    -------
    Named tuple with fields 'home', 'desktop', 'startmenu'

    Example:
    -------
    >>> from pyshortcuts import get_folders
    >>> folders = get_folders()
    >>> print("Home, Desktop, StartMenu ",
    ...       folders.home, folders.desktop, folders.startmenu)
    """
    UserFolders = namedtuple("UserFolders", ("home", "desktop", "startmenu"))
    return UserFolders(get_homedir(), get_desktop(), get_startmenu())


def make_shortcut(script, name=None, working_dir=None, description=None, icon=None,
                  folder=None, terminal=True, desktop=True,
                  startmenu=True, executable=None, noexe=False):
    """create shortcut

    Arguments:
    ---------
    script      (str) path to script, may include command-line arguments
    name        (str, None) name to display for shortcut [name of script]
    working_dir (str, None) directory where to run the script in
    description (str, None) longer description of script [`name`]
    icon        (str, None) path to icon file [python icon]
    folder      (str, None) subfolder of Desktop for shortcut [None] (See Note 1)
    terminal    (bool) whether to run in a Terminal [True]
    desktop     (bool) whether to add shortcut to Desktop [True]
    startmenu   (bool) whether to add shortcut to Start Menu [True] (See Note 2)
    executable  (str, None) name of executable to use [this Python] (see Note 3)
    noexe       (bool) whether to use no executable (script is entire command) [False]

    Notes:
    ------
    1. `folder` will place shortcut in a subfolder of Desktop and/or Start Menu
    2. Start Menu does not exist for Darwin / MacOSX
    3. executable defaults to the Python executable used to make shortcut.
    """
    from .shortcut import shortcut
    userfolders = get_folders()

    scut = shortcut(script, userfolders, name=name, description=description,
                    working_dir=working_dir, folder=folder, icon=icon)

    if noexe:
        full_script =scut.script
        executable = ''
    else:
        full_script =scut.full_script
        if executable is None:
            executable = get_pyexe()
        executable = Path(executable).resolve().as_posix()

        if Path(scut.full_script).resolve() == Path(executable).resolve():
            executable = ''

    # Check for other valid ways to run the script
    # try appending .exe if script itself not found
    if not Path(full_script).exists():
        tname = full_script + '.exe'
        if Path(tname).exists():
            executable = tname
            full_script = ''

    # If script is already executable use it directly instead of via pyexe
    ext = os.path.splitext(full_script)[1].lower()
    known_exes = [e.lower() for e in os.environ['PATHEXT'].split(os.pathsep)]
    if ext != '.py' and ext in known_exes:
        executable = full_script
        full_script = ''
    full_script = ' '.join((full_script, scut.arguments))

    # If we're on Anaconda Python, we need to wrap this
    # script in a batch file that activates an environment
    if Path(sys.prefix, 'conda-meta').exists():
        runnerbat = f'envrunner-{conda_env}.bat'
        runner = Path(sys.prefix, 'Scripts', runnerbat).resolve().as_posix()
        with open(runner, 'w') as fh:
            fh.write(ENVRUNNER)
        time.sleep(0.05)
        full_script = f"{executable} {full_script}".strip()
        executable = runner

    for (create, ofolder) in ((desktop, scut.desktop_dir),
                             (startmenu, scut.startmenu_dir)):
        if create:
            if not Path(ofolder).exists():
                os.makedirs(ofolder)
            dest = Path(ofolder, scut.target).resolve().as_posix()

            _WSHELL = win32com.client.Dispatch("Wscript.Shell")
            wscript = _WSHELL.CreateShortCut(dest)
            wscript.Targetpath = f'"{executable}"'
            wscript.Arguments = full_script
            wscript.WorkingDirectory = scut.working_dir or userfolders.home
            wscript.WindowStyle = 0
            wscript.Description = scut.description
            wscript.IconLocation = scut.icon
            wscript.save()

    return scut



#!/usr/bin/env python
"""
Create desktop shortcuts for Linux
"""
import os
import sys
from pathlib import Path
from collections import namedtuple

from .utils import  get_pyexe, get_homedir

DESKTOP_FORM = """[Desktop Entry]
Name={name:s}
Type=Application
Path={workdir:s}
Comment={desc:s}
Terminal={term:s}
Icon={icon:s}
Exec={execstring:s}
"""

def get_desktop():
    "get desktop location"
    homedir = get_homedir()
    desktop = Path(homedir, 'Desktop').resolve().as_posix()

    if sys.platform.startswith('linux'):
        # search for .config/user-dirs.dirs in HOMEDIR
        ud_file = Path(homedir, '.config', 'user-dirs.dirs').resolve().as_posix()
        if Path(ud_file).exists():
            val = desktop
            with open(ud_file, 'r') as fh:
                text = fh.readlines()
            for line in text:
                if 'DESKTOP' in line:
                    line = line.replace('$HOME', homedir)[:-1]
                    _, val = line.split('=')
                    val = val.replace('"', '').replace("'", "")
            desktop = val
    return desktop

def get_startmenu():
    "get start menu location"
    homedir = get_homedir()
    return Path(homedir, '.local', 'share', 'applications').resolve().as_posix()


def get_folders():
    """get user-specific folders

    Returns:
    -------
    Named tuple with fields 'home', 'desktop', 'startmenu'

    Example:
    -------
    >>> from pyshortcuts import get_folders
    >>> folders = get_folders()
    >>> print("Home, Desktop, StartMenu ",
    ...       folders.home, folders.desktop, folders.startmenu)
    """
    UserFolders = namedtuple("UserFolders", ("home", "desktop", "startmenu"))
    return UserFolders(get_homedir(), get_desktop(), get_startmenu())


def make_shortcut(script, name=None, description=None, icon=None, working_dir=None,
                  folder=None, terminal=True, desktop=True,
                  startmenu=True, executable=None, noexe=False):
    """create shortcut

    Arguments:
    ---------
    script      (str) path to script, may include command-line arguments
    name        (str, None) name to display for shortcut [name of script]
    description (str, None) longer description of script [`name`]
    icon        (str, None) path to icon file [python icon]
    working_dir (str, None) directory where to run the script in
    folder      (str, None) subfolder of Desktop for shortcut [None] (See Note 1)
    terminal    (bool) whether to run in a Terminal [True]
    desktop     (bool) whether to add shortcut to Desktop [True]
    startmenu   (bool) whether to add shortcut to Start Menu [True] (See Note 2)
    executable  (str, None) name of executable to use [this Python] (see Note 3)
    noexe       (bool) whether to use no executable (script is entire command) [False]

    Notes:
    ------
    1. `folder` will place shortcut in a subfolder of Desktop and/or Start Menu
    2. Start Menu does not exist for Darwin / MacOSX
    3. executable defaults to the Python executable used to make shortcut.
    """
    from .shortcut import shortcut

    userfolders = get_folders()
    if working_dir is None:
        working_dir = userfolders.home
    scut = shortcut(script, userfolders, name=name, description=description,
                    working_dir=working_dir, folder=folder, icon=icon)

    if noexe:
        full_script =scut.script
        executable = ''
    else:
        full_script =scut.full_script
        if executable is None:
            executable = get_pyexe()
        executable = Path(executable).resolve().as_posix()

        if Path(scut.full_script).resolve() == Path(executable).resolve():
            executable = ''

    execstring=f"{executable:s} {full_script:s} {scut.arguments:s}".strip()

    text = DESKTOP_FORM.format(name=scut.name, desc=scut.description,
                               workdir=scut.working_dir,
                               execstring=execstring,
                               icon=scut.icon,
                               term='true' if terminal else 'false')

    for (create, ofolder) in ((desktop, scut.desktop_dir),
                             (startmenu, scut.startmenu_dir)):
        if create:
            if not Path(ofolder).exists():
                os.makedirs(ofolder)
            dest = Path(ofolder, scut.target).resolve().as_posix()
            with open(dest, 'w') as fout:
                fout.write(text)
            os.chmod(dest, 493) ## = octal 755 / rwxr-xr-x
    return scut



import sys
from time import sleep, time
from datetime import datetime

from optparse import OptionParser

usage = "Usage: timer.py [options]"

parser = OptionParser(usage=usage, prog="timer", version="simple clock")

parser.add_option("-t", "--time", dest="maxtime", metavar='maxtime',
                      default=None, help="maximum run time (seconds)")
parser.add_option("-u", "--update", dest="update", metavar='update',
                      default=0.1, help="update time (seconds)")

(options, args) = parser.parse_args()

update = float(options.update)
maxtime = options.maxtime
if maxtime is not None:
    maxtime = float(maxtime)

t0 = time()

while True:
    sleep(update)
    print("%s : %.3f " % (datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f'), time()-t0))
    sys.stdout.flush()
    if maxtime is not None and (time()-t0 > maxtime):
        break

print("done.")
sleep(0.5)



#!/usr/bin/env python
"""
pyshortcuts
"""
import os
import sys
from pathlib import Path
from argparse import ArgumentParser, RawDescriptionHelpFormatter

from .version import version as __version__
from .utils import (fix_filename, new_filename, fix_varname, isotime,
                    get_pyexe, bytes2str, str2bytes, get_homedir, get_cwd,
                    uname, scut_ext, ico_ext)

from .gformat import gformat
from .debugtimer import debugtimer

make_shortcut =  get_folders = None
if uname.startswith('lin'):
    from .linux import make_shortcut, get_folders
elif uname.startswith('darwin'):
    from .darwin import make_shortcut, get_folders
elif uname.startswith('win'):
    from .windows import make_shortcut, get_folders

# back compat
platform = uname

from .shortcut import shortcut, Shortcut

try:
    import wx
    from .wxgui import ShortcutFrame
except ImportError:
    ShortCutFrame = None


def get_desktop():
    "for back compatibility"
    return get_folders().desktop

def shortcut_cli():
    '''
    command-line interface to creating desktop shortcuts
    '''
    desc = 'create desktop and start menu shortcuts'

    parser = ArgumentParser(description=desc,
                            formatter_class=RawDescriptionHelpFormatter)


    parser.add_argument('-v', '--version', dest='version', action='store_true',
                        default=False, help='show version')

    parser.add_argument('-n', '--name', dest='name', default=None,
                        help='name for shortcut link')

    parser.add_argument('-i', '--icon', dest='icon', default=None,
                        help='name of icon file')

    parser.add_argument('-f', '--folder', dest='folder', default=None,
                        help='subfolder on desktop to put shortcut')

    parser.add_argument('-e', '--executable', dest='exe', default=None,
                        help='name of executable to use (python)')

    parser.add_argument('-x', '--no-executable', dest='noexe', action='store_true',
                        default=False, help='use no implied executable [False]')

    parser.add_argument('-t', '--terminal', dest='terminal', action='store_true',
                        default=True, help='run script in a Terminal [True]')

    parser.add_argument('-g', '--gui', dest='gui', action='store_true',
                        default=False, help='run script as GUI, with no Terminal [False]')

    parser.add_argument('-d', '--desktop', dest='desktop', action='store_true',
                        default=True, help='create desktop shortcut [True]')

    parser.add_argument('-s', '--startmenu', dest='startmenu', action='store_true',
                        default=True, help='create Start Menu shortcut [True]')

    parser.add_argument('-w', '--wxgui', dest='wxgui', action='store_true',
                        default=False, help='run GUI version of pyshortcut [False]')

    parser.add_argument('-b', '--bootstrap', dest='bootstrap', action='store_true',
                        default=False, help='make desktop shortcut to wxGUI')

    parser.add_argument('scriptname', nargs='?',
                        help='script name, including arguments')

    args = parser.parse_args()

    if args.version:
        print(f"pyshortcut {__version__}")

    if (args.wxgui or args.bootstrap) and ShortcutFrame is None:
        print("wxpython is required to run GUI")
        sys.exit()

    if args.bootstrap:
        bindir = 'bin'
        if uname.startswith('win'):
            bindir = 'Scripts'
        fpath = Path(__file__)
        icon = Path(fpath.parent, 'icons', f'ladder.{ico_ext[0]}'
                        ).resolve().as_posix()
        script = Path(sys.prefix, bindir, 'pyshortcut').resolve().as_posix()
        make_shortcut(f"{script} --wxgui", name='PyShortcut',
                      terminal=False, icon=icon)

    elif args.wxgui:
        app = wx.App()
        ShortcutFrame().Show(True)
        app.MainLoop()

    else:
        if args.gui:
            args.terminal = False

        if args.scriptname is None:
            print("pyshortcut: must provide one script.  try 'pyshortcut -h'")
        else:
            p_icon = Path(args.icon).resolve()
            if p_icon.exists():
                icon = p_icon.resolve().as_posix()
            else:
                parent = p_icon.parent
                stem  = p_icon.stem
                for ext in ico_ext:
                    x = Path(parent, f"{stem}.{ext}").absolute()
                    if x.exists():
                        icon = x.resolve().as_posix()
            make_shortcut(args.scriptname, name=args.name,
                          terminal=args.terminal, folder=args.folder,
                          icon=icon, desktop=args.desktop,
                          startmenu=args.startmenu, executable=args.exe,
                          noexe=args.noexe)



#!/usr/bin/env python
"""
utilities for pyshortcuts
"""
import os
import sys
from pathlib import Path
from datetime import datetime
from string import ascii_letters

try:
    from pwd import getpwnam
except ImportError:
    getpwnam = None

uname = "unknown"
scut_ext = "lnk"
ico_ext = ("ico",)

if sys.platform.startswith('lin'):
    uname = "linux"
    scut_ext = "desktop"
    ico_ext = ("ico", "png")
elif sys.platform.startswith('darwin'):
    uname = 'darwin'
    scut_ext = "app"
    ico_ext = ("icns",)
elif sys.platform.startswith('win')  or os.name.startswith('nt'):
    uname = "win"
    scut_ext = "lnk"
    ico_ext = ("ico",)

def get_homedir():
    "determine home directory"
    # for Unixes, allow for sudo case
    susername = os.environ.get("SUDO_USER", None)
    if susername is not None and getpwnam is not None:
        return Path(getpwnam(susername).pw_dir).resolve().as_posix()

    homedir = Path.home()

    # For Windows, ask for parent of Roaming 'Application Data' directory
    if homedir is None and os.name == 'nt':
        try:
            from win32com.shell import shellcon, shell
            homedir = Path(shell.SHGetFolderPath(0, shellcon.CSIDL_APPDATA, 0, 0))
        except ImportError:
            pass

    # try the HOME environmental variable
    if homedir is  None:
        test = os.path.expandvars('$HOME')
        if test not in (None, '$HOME'):
            homedir = Path(test)

    # finally, use current folder
    if homedir is None:
        homedir = Path('.')
    return homedir.resolve().as_posix()

def get_cwd():
    """get current working directory
    Note: os.getcwd() can fail with permission error.

    when that happens, this changes to the users `HOME` directory
    and returns that directory so that it always returns an existing
    and readable directory.
    """
    try:
        return Path('.').resolve().as_posix()
    except Exception:
        home = get_homedir()
        os.chdir(home)
        return home


def isotime(dtime=None, timespec='seconds', sep=' '):
    """return ISO format of current timestamp:
          2024-04-27 17:31:12
    """
    if dtime is None:
        dtime = datetime.now()
    return datetime.isoformat(dtime, timespec=timespec, sep=sep)

BAD_FILECHARS = ';~,`!%$@$&^?*#:"/|\'\\\t\r\n(){}[]<>'
GOOD_FILECHARS = '_'*len(BAD_FILECHARS)
TRANS_FILE = str.maketrans(BAD_FILECHARS, GOOD_FILECHARS)

def fix_filename(filename):
    """
    fix string to be a 'good' filename, with very few special
    characters and (optionally) no more than 1 '.'.

    More restrictive than most OSes, but avoids hard-to-deal with filenames
    """
    fname = str(filename).translate(TRANS_FILE)
    if fname.count('.') > 1:
        words = fname.split('.')
        ext = words.pop()
        fname = f"{'_'.join(words)}.{ext}"
    return fname

def fix_varname(varname):
    """fix string to be a 'good' variable name."""
    vname = fix_filename(varname)
    if vname[0] not in (ascii_letters+'_'):
        vname = f'_{vname}'
    for c in '=+-.':
        vname = vname.replace(c, '_')
    while vname.endswith('_'):
        vname = vname[:-1]
    while '__' in vname:
        vname = vname.replace('__', '_')
    return vname

def get_pyexe():
    "python executable"
    return Path(sys.executable).resolve().as_posix()

def new_filename(filename):
    """
    increment filename to be an unused filename
    """
    fpath = Path(filename)
    if fpath.exists():
        fstem = fpath.stem
        fsuffix = fpath.suffix
        if fsuffix.startswith('.'):
            fsuffix = fsuffix[1:]
        if len(fsuffix) == 0:
            fsuffix = 'txt'
        int_suffix = True
        fsint = 0
        try:
            fsint = int(fsuffix)
        except (ValueError, TypeError):
            int_suffix = False
        while fpath.exists():
            fsint += 1
            if int_suffix:
                fpath = Path(f"{fstem}.{fsint:03d}")
            else:
                if '_' in fstem:
                    w = fstem.split('_')
                    try:
                        fsint = int(w[-1])
                        fstem = '_'.join(w[:-1])
                    except (ValueError, TypeError):
                        pass
                fpath = Path(f"{fstem}_{fsint:03d}.{fsuffix}")

    return fpath.as_posix()

def bytes2str(s):
    'byte to string conversion'
    if isinstance(s, str):
        return s
    if isinstance(s, bytes):
        return s.decode(sys.stdout.encoding)
    return str(s, sys.stdout.encoding)

def str2bytes(s):
    'string to byte conversion'
    if isinstance(s, bytes):
        return s
    return bytes(s, sys.stdout.encoding)



#!/usr/bin/env python
"""
Create desktop shortcuts for Darwin / MacOS
"""
import os
import sys
import shutil
from pathlib import Path
from collections import namedtuple

from .utils import  get_pyexe, get_homedir


def get_startmenu():
    "get start menu location"
    return ''

def get_desktop():
    "get desktop location"
    return Path(get_homedir(), 'Desktop').resolve().as_posix()

def get_folders():
    """get user-specific folders

    Returns:
    -------
    Named tuple with fields 'home', 'desktop', 'startmenu'

    Example:
    -------
    >>> from pyshortcuts import get_folders
    >>> folders = get_folders()
    >>> print("Home, Desktop, StartMenu ",
    ...       folders.home, folders.desktop, folders.startmenu)
    """
    UserFolders = namedtuple("UserFolders", ("home", "desktop", "startmenu"))
    return UserFolders(get_homedir(), get_desktop(), get_startmenu())


def make_shortcut(script, name=None, description=None, icon=None, working_dir=None,
                  folder=None, terminal=True, desktop=True,
                  startmenu=False, executable=None, noexe=False):
    """create shortcut

    Arguments:
    ---------
    script      (str) path to script, may include command-line arguments
    name        (str, None) name to display for shortcut [name of script]
    description (str, None) longer description of script [`name`]
    icon        (str, None) path to icon file [python icon]
    working_dir (str, None) directory where to run the script in
    folder      (str, None) subfolder of Desktop for shortcut [None] (See Note 1)
    terminal    (bool) whether to run in a Terminal [True]
    desktop     (bool) whether to add shortcut to Desktop [True]
    startmenu   (bool) whether to add shortcut to Start Menu [False] (See Note 2)
    executable  (str, None) name of executable to use [this Python] (see Note 3)
    noexe       (bool) whether to use no executable (script is entire command) [False]

    Notes:
    ------
    1. `folder` will place shortcut in a subfolder of Desktop and/or Start Menu
    2. Start Menu does not exist for Darwin / MacOSX
    3. executable defaults to the Python executable used to make shortcut.
    """
    if not desktop:
        return None

    userfolders = get_folders()
    if working_dir is None:
        working_dir = ''

    from .shortcut import shortcut


    scut = shortcut(script, userfolders, name=name, description=description,
                    working_dir=working_dir, folder=folder, icon=icon)

    if noexe:
        full_script =scut.script
        executable = ''
    else:
        full_script =scut.full_script
        if executable is None:
            executable = get_pyexe()
        executable = Path(executable).resolve().as_posix()
        if Path(scut.full_script).resolve() == Path(executable).resolve():
            executable = ''

    if not Path(scut.desktop_dir).exists():
        os.makedirs(scut.desktop_dir)

    osascript = f'{full_script} {scut.arguments}'
    osascript = osascript.replace(' ', '\\ ')

    dest = Path(scut.desktop_dir, scut.target).resolve().as_posix()

    if Path(dest).exists():
        shutil.rmtree(dest)

    os.mkdir(dest)
    os.mkdir(Path(dest, 'Contents'))
    os.mkdir(Path(dest, 'Contents', 'MacOS'))
    os.mkdir(Path(dest, 'Contents', 'Resources'))

    opts = {'name': scut.name,
            'desc': scut.description,
            'script': full_script,
            'workdir': scut.working_dir,
            'args': scut.arguments,
            'prefix': Path(sys.prefix).resolve().as_posix(),
            'exe': executable,
            'osascript': osascript}

    info = """<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN"
"http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
  <dict>
  <key>CFBundleGetInfoString</key> <string>{desc:s}</string>
  <key>CFBundleName</key> <string>{name:s}</string>
  <key>CFBundleExecutable</key> <string>{name:s}</string>
  <key>CFBundleIconFile</key> <string>{name:s}</string>
  <key>CFBundlePackageType</key> <string>APPL</string>
  </dict>
</plist>
"""

    text = ['#!/bin/bash',
            "export EXE={exe:s}",
            "export SCRIPT={script:s}",
            "export ARGS='{args:s}'"]

    if scut.working_dir not in (None, ''):
        text.append("cd {workdir:s}")

    if terminal:
        text.append(r"""osascript -e 'tell application "Terminal"
   do script "'${{EXE}}\ {osascript:s}'"
end tell
'
""")
    else:
        text.append("$EXE $SCRIPT $ARGS")

    text.append('\n')
    text = '\n'.join(text)

    with open(Path(dest, 'Contents', 'Info.plist'), 'w') as fout:
        fout.write(info.format(**opts))

    ascript_name = Path(dest, 'Contents', 'MacOS', scut.name).as_posix()
    with open(ascript_name, 'w') as fout:
        fout.write(text.format(**opts))

    os.chmod(ascript_name, 493)  # = octal 755 / rwxr-xr-x
    icon_dest = Path(dest, 'Contents', 'Resources', scut.name + '.icns').as_posix()
    shutil.copy(scut.icon, icon_dest)
    return scut



"""fixed length formatting of floats"""

from math import log10

def getfloat_attr(obj, attr, length=11):
    """Format an attribute of an object for printing."""
    val = getattr(obj, attr, None)
    if val is None:
        return 'unknown'
    if isinstance(val, int):
        return f'{val}'
    if isinstance(val, float):
        return gformat(val, length=length).strip()
    return repr(val)


def gformat(val, length=11):
    """Format a number with '%g'-like format.

    Except that:
        a) the length of the output string will be of the requested length.
        b) positive numbers will have a leading blank.
        c) the precision will be as high as possible.
        d) trailing zeros will not be trimmed.

    The precision will typically be ``length-7``.

    Parameters
    ----------
    val : float
        Value to be formatted.
    length : int, optional
        Length of output string (default is 11).

    Returns
    -------
    str
        String of specified length.

    Notes
    ------
    Positive values will have leading blank.

    """


    if val is None or isinstance(val, bool):
        return f'{repr(val):>{length}s}'
    try:
        expon = int(log10(abs(val)))
    except (OverflowError, ValueError):
        expon = 0
    length = max(length, 7)
    form = 'e'
    prec = length - 7
    ab_expon = abs(expon)
    if ab_expon > 99:
        prec -= 1
    elif ((expon >= 0 and expon < (prec+4))
          or (expon <= -1 and -expon < (prec-2))
          or (expon <= -1 and prec < 5 and abs(expon)<3 )):
        form = 'f'
        prec += 4
        if expon > 0:
            prec -= expon

    def fmt(val, length, prec, form):
        "format "
        prec = max(prec, 0)
        out = f'{val:{length}.{prec}{form}}'
        if form == 'e' and 'e+0' in out or 'e-0' in out:
            out = f'{val:{length+1}.{prec+1}{form}}'.replace('e-0', 'e-').replace('e+0', 'e+')

        return out

    prec += 1
    out = '_'*(length+2)
    while len(out) > length:
        prec -= 1
        out = fmt(val, length, prec, form)
    if '_' in out:
        out = fmt(val, length, prec, form)
    while len(out) < length:
        prec += 1
        out = fmt(val, length, prec, form)
    return out


def fcompact(val):
    """format a float value, removing extra trailing zeros"""
    val = f'{val:.6f}'
    while val.endswith('0'):
        val = val[:-1]
    if val.endswith('.'):
        val = val + '0'
    return val



#!/usr/bin/env python
"""
shortcut function
"""
from pathlib import Path
from collections import namedtuple

from .utils import get_pyexe, fix_filename, scut_ext, ico_ext

Shortcut = namedtuple("Shortcut", ('name', 'description', 'icon', 'target',
                      'working_dir', 'script', 'full_script', 'arguments',
                      'desktop_dir', 'startmenu_dir'))


def shortcut(script, userfolders, name=None, description=None,
             folder=None, working_dir=None, icon=None):
    """representation of a Shortcuts parameters.

    Arguments:
    ----------
    script        script to run, may include optional arguments
    userfolders   userfolders returned from the appropriate get_userfolders()
    name          name for shortcut (`None` to use name of script file)
    description   long description (`None` to use name of script file)
    folder        sub-folder of Desktop to place shortcut (`None` for on Desktop)
    working_dir   directory where to run the script in
    icon          full path to icon file for shortcut (`None` for default)

    Returns:
    --------
    Namedtuple with the following fields:

      name           name for shortcut
      description    long description of shortcut
      icon           path for icon file
      target         name of the shortcut file (without folder name)
      working_dir    directory where to run the script in
      script         shortname of python script to be run (without arguments)
      full_script    full path of python script to be run (without arguments)
      arguments      command line arguments
      desktop_dir    full path of desktop folder (may need to be created)
      startmenu_dir  full path of startmenu folder (may need to be created)

    """
    if not isinstance(script, str) or len(script) < 1:
        raise ValueError("`script` for shortcut must be a non-zero length string")

    words = script.strip().split(' ', 1)
    if len(words) < 2:
        words.append('')

    script, arguments = words[0], words[1]
    if script in ('_', '{}'):
        script = words[0] = get_pyexe()

    full_script = Path(script).resolve().as_posix()

    if name is None:
        name = script

    name = fix_filename(Path(name).stem)
    if name.endswith('.py'):
        name = name[:-3]

    if description is None:
        description = name

    target = f'{name}.{scut_ext}'
    if icon is not None and len(str(icon)) > 0:
        picon = Path(icon).resolve()
        if not picon.exists():
            for ext in ico_ext:
                ticon = Path(f"{icon:s}.{ext:s}").resolve()
                if ticon.exists():
                    picon = ticon
                    break

    if icon is None or not Path(icon).exists():
        _parent = Path(__file__).parent
        icon = f'py.{ico_ext[0]:s}'
        icon = Path(_parent, 'icons', icon).resolve().as_posix()

    desktop_dir = userfolders.desktop
    if folder is not None:
        if folder.startswith(desktop_dir):
            folder = folder[len(desktop_dir)+1:]
        desktop_dir = Path(desktop_dir, folder).resolve().as_posix()

    startmenu_dir = userfolders.startmenu
    if folder is not None and len(startmenu_dir) > 1:
        if folder.startswith(startmenu_dir):
            folder = folder[len(startmenu_dir)+1:]
        startmenu_dir = Path(startmenu_dir, folder).resolve().as_posix()

    return Shortcut(name, description, icon, target, working_dir,
                    script, full_script, arguments, desktop_dir,
                    startmenu_dir)
